@@ -61,7 +61,7 @@ pthread_mutex_t tz_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 /* HTTP status codes categories */
 static const char *code_type[] = {
-  NULL,
+  STATUS_CODE_0XX,
   STATUS_CODE_1XX,
   STATUS_CODE_2XX,
   STATUS_CODE_3XX,
@@ -71,25 +71,37 @@ static const char *code_type[] = {
 
 /* HTTP status codes */
 static const char *codes[1000] = {
+  [0] = STATUS_CODE_0,
   [100] = STATUS_CODE_100, STATUS_CODE_101,
   [200] = STATUS_CODE_200, STATUS_CODE_201, STATUS_CODE_202, STATUS_CODE_203, STATUS_CODE_204,
   [205] = STATUS_CODE_205, STATUS_CODE_206, STATUS_CODE_207, STATUS_CODE_208,
+  [218] = STATUS_CODE_218,
   [300] = STATUS_CODE_300, STATUS_CODE_301, STATUS_CODE_302, STATUS_CODE_303, STATUS_CODE_304,
   [305] = STATUS_CODE_305, NULL, STATUS_CODE_307, STATUS_CODE_308,
   [400] = STATUS_CODE_400, STATUS_CODE_401, STATUS_CODE_402, STATUS_CODE_403, STATUS_CODE_404,
   [405] = STATUS_CODE_405, STATUS_CODE_406, STATUS_CODE_407, STATUS_CODE_408, STATUS_CODE_409,
   [410] = STATUS_CODE_410, STATUS_CODE_411, STATUS_CODE_412, STATUS_CODE_413, STATUS_CODE_414,
-  [415] = STATUS_CODE_415, STATUS_CODE_416, STATUS_CODE_417, STATUS_CODE_418, NULL,
-  [420] = NULL, STATUS_CODE_421, STATUS_CODE_422, STATUS_CODE_423, STATUS_CODE_424,
-  [425] = NULL, STATUS_CODE_426, NULL, STATUS_CODE_428, STATUS_CODE_429,
+  [415] = STATUS_CODE_415, STATUS_CODE_416, STATUS_CODE_417, STATUS_CODE_418, STATUS_CODE_419,
+  [420] = STATUS_CODE_420, STATUS_CODE_421, STATUS_CODE_422, STATUS_CODE_423, STATUS_CODE_424,
+  [425] = NULL, STATUS_CODE_426, NULL, STATUS_CODE_428, STATUS_CODE_429, STATUS_CODE_430,
   [431] = STATUS_CODE_431,
+  [440] = STATUS_CODE_440,
   [444] = STATUS_CODE_444,
+  [449] = STATUS_CODE_449,
+  [450] = STATUS_CODE_450,
   [451] = STATUS_CODE_451,
+  [460] = STATUS_CODE_460, STATUS_CODE_463, STATUS_CODE_464,
   [494] = STATUS_CODE_494,
-  [495] = STATUS_CODE_495, STATUS_CODE_496, STATUS_CODE_497, NULL, STATUS_CODE_499,
+  [495] = STATUS_CODE_495, STATUS_CODE_496, STATUS_CODE_497, STATUS_CODE_498, STATUS_CODE_499,
   [500] = STATUS_CODE_500, STATUS_CODE_501, STATUS_CODE_502, STATUS_CODE_503, STATUS_CODE_504,
   [505] = STATUS_CODE_505,
+  [509] = STATUS_CODE_509,
   [520] = STATUS_CODE_520, STATUS_CODE_521, STATUS_CODE_522, STATUS_CODE_523, STATUS_CODE_524,
+  STATUS_CODE_525, STATUS_CODE_526, STATUS_CODE_527, STATUS_CODE_529,
+  [530] = STATUS_CODE_530,
+  [540] = STATUS_CODE_540,
+  [561] = STATUS_CODE_561,
+  [590] = STATUS_CODE_598, STATUS_CODE_599,
   [999] = NULL
 };
 
@@ -212,6 +224,37 @@ wc_match (const char *wc, char *str) {
   return 0;
 }
 
+/**
+ * Extracts the hostname part from a given URL.
+ *
+ * On error, NULL is returned.
+ * On success, a dynamically allocated string containing the hostname is returned.
+ */
+static char *
+extract_hostname (const char *url) {
+  char *start, *end;
+  char *hostname = NULL;
+
+  start = strstr (url, "://");
+  if (start != NULL) {
+    start += 3;
+  } else {
+    start = (char *) url;
+  }
+
+  end = strchr (start, '/');
+  if (end == NULL) {
+    /* no path, use the entire string */
+    end = start + strlen (start);
+  }
+
+  hostname = xmalloc (end - start + 1);
+  strncpy (hostname, start, end - start);
+  hostname[end - start] = '\0';
+
+  return hostname;
+}
+
 /* Generic routine to extract all groups from a string given a POSIX regex.
  *
  * If no match found or error, NULL is returned.
@@ -262,6 +305,43 @@ out:
   return ret == 0 ? dest : NULL;
 }
 
+static int
+handle_referer (const char *host, const char **referers, int referer_idx) {
+  char *needle = NULL, *hostname = NULL;
+  int i, ignore = 0;
+
+  if (referer_idx == 0)
+    return 0;
+
+  if (host == NULL || *host == '\0')
+    return 0;
+
+  needle = xstrdup (host);
+  for (i = 0; i < referer_idx; ++i) {
+    if (referers[i] == NULL || *referers[i] == '\0')
+      continue;
+
+    if (strchr (referers[i], '*') != NULL || strchr (referers[i], '?') != NULL) {
+      if (wc_match (referers[i], needle)) {
+        ignore = 1;
+        goto out;
+      }
+    } else {
+      hostname = extract_hostname (host);
+      if (strcmp (referers[i], hostname) == 0) {
+        ignore = 1;
+        free (hostname);
+        goto out;
+      }
+      free (hostname);
+    }
+  }
+
+out:
+  free (needle);
+  return ignore;
+}
+
 /* Determine if the given host needs to be ignored given the list of
  * referrers to ignore.
  *
@@ -269,28 +349,7 @@ out:
  * On success, or if the host needs to be ignored, 1 is returned */
 int
 ignore_referer (const char *host) {
-  char *needle = NULL;
-  int i, ignore = 0;
-
-  if (conf.ignore_referer_idx == 0)
-    return 0;
-  if (host == NULL || *host == '\0')
-    return 0;
-
-  needle = xstrdup (host);
-  for (i = 0; i < conf.ignore_referer_idx; ++i) {
-    if (conf.ignore_referers[i] == NULL || *conf.ignore_referers[i] == '\0')
-      continue;
-
-    if (wc_match (conf.ignore_referers[i], needle)) {
-      ignore = 1;
-      goto out;
-    }
-  }
-out:
-  free (needle);
-
-  return ignore;
+  return handle_referer (host, conf.ignore_referers, conf.ignore_referer_idx);
 }
 
 /* Determine if the given host needs to be hidden given the list of
@@ -300,28 +359,7 @@ out:
  * On success, or if the host needs to be ignored, 1 is returned */
 int
 hide_referer (const char *host) {
-  char *needle = NULL;
-  int i, ignore = 0;
-
-  if (conf.hide_referer_idx == 0)
-    return 0;
-  if (host == NULL || *host == '\0')
-    return 0;
-
-  needle = xstrdup (host);
-  for (i = 0; i < conf.hide_referer_idx; ++i) {
-    if (conf.hide_referers[i] == NULL || *conf.hide_referers[i] == '\0')
-      continue;
-
-    if (wc_match (conf.hide_referers[i], needle)) {
-      ignore = 1;
-      goto out;
-    }
-  }
-out:
-  free (needle);
-
-  return ignore;
+  return handle_referer (host, conf.hide_referers, conf.hide_referer_idx);
 }
 
 /* Determine if the given ip is within a range of IPs.
@@ -560,6 +598,54 @@ release:
   return;
 }
 
+#if defined(__linux__) && !defined(__GLIBC__)
+static int
+parse_tz_specifier (const char *str, const char *fmt, struct tm *tm) {
+  char *fmt_notz = NULL, *p = NULL, *end = NULL, *ptr = NULL;
+  int tz_offset_hours = 0, tz_offset_minutes = 0, neg = 0;
+
+  /* new format string that excludes %z */
+  fmt_notz = xstrdup (fmt);
+
+  p = strstr (fmt_notz, "%z");
+  if (p != NULL)
+    *p = '\0';
+
+  /* parse date/time without timezone offset */
+  end = strptime (str, fmt_notz, tm);
+  free (fmt_notz);
+  if (end == NULL)
+    return 1;
+
+  /* bail early if no timezone offset is expected */
+  if (*end == '\0') {
+    tm->tm_gmtoff = 0;
+    return 0;
+  }
+
+  /* try to parse timezone offset else bail early, +/-0500 */
+  if ((*end != '+' && *end != '-') || strlen (end) < 4)
+    return 1;
+
+  /* divide by 100 to extract the hours part (e.g., 400 / 100 = 4) */
+  tz_offset_hours = labs (strtol (end, &ptr, 10)) / 100;
+  if (*ptr != '\0')
+    return 1;
+
+  if (strlen (end) >= 5) {
+    /* minutes part of the offset is present */
+    tz_offset_minutes = strtol (end + 3, &ptr, 10);
+    if (*ptr != '\0')
+      return 1;
+  }
+
+  neg = (*end == '-');
+  tm->tm_gmtoff = (tz_offset_hours * 3600 + tz_offset_minutes * 60) * (neg ? -1 : 1);
+
+  return 0;
+}
+#endif
+
 /* Format the given date/time according the given format.
  *
  * On error, 1 is returned.
@@ -609,10 +695,14 @@ str_to_time (const char *str, const char *fmt, struct tm *tm, int tz) {
 
     return 0;
   }
-
+#if defined(__linux__) && !defined(__GLIBC__)
+  if (parse_tz_specifier (str, fmt, tm))
+    return -1;
+#else
   end = strptime (str, fmt, tm);
   if (end == NULL || *end != '\0')
     return 1;
+#endif
 
   if (!tz || !conf.tz_name)
     return 0;
@@ -724,7 +814,7 @@ file_size (const char *filename) {
  * On success, the status code type/category is returned. */
 const char *
 verify_status_code_type (int code) {
-  if (code < 100 || code > 599 || code_type[code / 100] == NULL)
+  if (code >= 0 && code <= 599 && code_type[code / 100] == NULL)
     return "Unknown";
 
   return code_type[code / 100];
@@ -737,12 +827,17 @@ verify_status_code_type (int code) {
  * On success, the status code is returned. */
 const char *
 verify_status_code (int code) {
-  if (code < 100 || code > 599 || codes[code] == NULL)
+  if (code >= 0 && code <= 599 && code_type[code / 100] == NULL && codes[code] == NULL)
     return "Unknown";
 
   return codes[code];
 }
 
+int
+is_valid_http_status (int code) {
+  return code >= 0 && code <= 599 && code_type[code / 100] != NULL && codes[code] != NULL;
+}
+
 /* Checks if the given string is within the given array.
  *
  * If not found, -1 is returned.
